/*
 InsertToc by Katja Brunner <https://katjabrunner.de/wissen/javascript/inserttoc.html

 @version		1.0
 @dependencies: jQuery <http://jquery.com>

 Inserts a linked table of contents with anchors to all <h2> of the callee.
 The toc won't be created if there is only one <h2>.

 @option at:       The selector of the element to insert the toc.
 @option position: Where to put the toc in relative to 'at' element: "after" | "before" | "intoAsFirst" | "intoAsLast"
*/
(function($) {

    $.fn.insertToc = function(params) {
        let options = {
            contentSelectors: ['h2'],
            at: 'h1:first',
            position: 'after'
        };

        if (params) {
            $.extend(options, params);
        }

        return this.each(function() {
            let toc = createToc(this, options.contentSelectors);
            if (toc) {
                switch (options.position) {
                    case 'intoAsFirst':
                        $(toc).prependTo(options.at); break;
                    case 'intoAsLast':
                        $(toc).appendTo(options.at); break;
                    case 'before':
                        $(toc).insertBefore(options.at); break;
                    case 'after':
                    default:
                        $(toc).insertAfter(options.at); break;
                }
            }
        });

        /**
         * Creates the html of the toc.
         *
         * @param {HTMLElement} context The context to find the headings in.
         * @param {<string>[]} contentSelectors The heading selectors.
         *
         * @returns {string} The html string of the toc.
         */
        function createToc(context, contentSelectors) {
            let toc = null;
            let h2 = $(context).find(contentSelectors[0]);

            if (h2.length > 1) {
                toc = `<ul class="toc">`;
                h2.each(function(index, elem) {
                    let jElem = $(elem);
                    let text = jElem.text();
                    let id = jElem.attr('id');
                    if (!id) {
                        id = 'toc_' + text.replace(/[^a-zA-Z0-9]/g,'-');
                        jElem.attr('id', id);
                    }

                    toc += `<li><a href="${ location.pathname }#${ id }">${ jElem.text() }</a></li>`
                });
                toc += `</ul>`;
            }
            return toc;
        }
    };

}(jQuery));